package fi.helsinki.cs.koskelo.displayer;

import fi.hy.eassari.showtask.trainer.CacheException;

/**
 * Luokka, joka huolehtii staattisten tehtvien
 * html-koodin tuottamisesta.
 * @author Tom Bertell, Koskelo-projekti
 */

public class FillInTTK91Displayer extends CommonDisplayer{

	/**
	 * Konstruktori, kutsuu ylluokan CommonDisplayerin
	 * konstruktoria.
	 */

	public FillInTTK91Displayer(){
		super();
	}

	final int ROWS = 12;    // vastauslaatikon korkeus 
	final int COLS = 50;    // vastauslaatikon leveys
	final String TARGETSERVLET = "Answer2.do2"; // kynnistettv servletti
    
	/**
	 * Tekee tehtvn tiedoista html-koodia ja palauttaa 
	 * html-koodin String-oliona. Kutsutaan DisplayerServlet2- ja
	 * Answer2-servleteist.
	 *
	 * @param  initval tehtvn vastaus
	 * @param  params  generoidut parametrit
	 * @param  hiddens lomakkeen piilokentt String-oliona.
	 * @param  allowTry kertoo tuleeko sivun loppuun vastauspainike  
	 * @return tehtvn esitys html-muodossa
	 * @throws CacheException jos tulee ongelmia attribuuttien 
	 *                        hakemisessa AttributeCachesta
	 */

	public String getSetting(String [] initVal, 
													 String params, 
													 String hiddens,  
													 boolean allowTry) throws CacheException{

		String taskDescription; // Tehtvnanto
		String input;           // syte
		String inputHeader;     // sytteen otsikko
		String exampleCode;     // malliratkaisu
		StringBuffer setting = new StringBuffer();  // kertn html-sivu
	  
		// Listn piilokenttiin tehtvn tyyppi.
		
		hiddens += ("<input type=\"hidden\" name=\"tasktype\"" 
								+"value=\"fillinttk91task\">");
		/*
		 * Haetaan tehtvnanto, sytteet, sytteen otsikko ja malliratkaisu.
		 */

		taskDescription = 
			cache.getAttribute("T", taskid, "taskDescription", language);
		if (taskDescription == null || taskDescription.equals("null"))
			taskDescription = "";
		input = 
			cache.getAttribute("T", taskid, "publicInput", language);
		if (input == null || input.equals("null"))
			input = "";
		inputHeader = 
			cache.getAttribute("D", "staticttk91taskdisplayer", 
												 "inputHeader", language);
		exampleCode =
			cache.getAttribute("T", taskid, "exampleCode", language);
		if (exampleCode == null || exampleCode.equals("null"))
			exampleCode = "";
	
		/*
		 * Lomake alkaa
		 */

		setting.append("<form action=" + TARGETSERVLET + 
									 " method=\"post\" name=\"fillinttk91task\"" +
									 "id=\"fillinttk91task\">");
	
    /*   
		 * Listn lomakkeeseen tehtvnanto ja sytteet
		 */

		setting.append(TTK91DisplayerUtils.getHTMLElementTask(taskDescription));
		setting.append(TTK91DisplayerUtils.getHTMLElementInput(inputHeader, input));

		/* 
		 * Listn ennen vastauslaatikkoa malliratkaisun osa, joka on 
		 * ennen aukkoa ja vastauslaatikon jlkeen osa, joka on aukon jlkeen. 
		 * Aukon paikalle laitetaan vastauslaatikko. Jos parametri initVal on null,
		 * luodaan tyhj vastauslaatikko, muutoin opiskelijan vastaus 
		 * laitetaan laatikkoon.
		 */

		setting.append(getHTMLElementBeforeEmpty(exampleCode));

		if (initVal == null) {
			setting.append(TTK91DisplayerUtils.
										 getHTMLElementAnswerBox(ROWS/2, COLS));
		} else {
			setting.append(TTK91DisplayerUtils.
										 getHTMLElementAnswerBox(initVal, ROWS/2, COLS));
		}
		
		setting.append(getHTMLElementAfterEmpty(exampleCode));
		
		/*
		 * Listn analyserin tarvitsemat piilokentt lomakkeeseen
		 */

		setting.append(hiddens);
		
		/*
		 * Jos tehtvn lhetys sallitaan, luodaan lhet-nappi
		 */

		if (allowTry) {
			setting.append(super.getButton());
		}
       
		/*
		 * Lomake loppuu
		 */

		setting.append("</form>");
		return new String(setting);

	}//getSetting
    
	/**
	 * Luo sen osan mallikoodista, joka tulee ennen aukkoa.
	 *
	 * @param exampleCode mallivastaus
	 * @return elementti html-muodossa
	 */  

	private String getHTMLElementBeforeEmpty(String exampleCode){
		
		String beginSeparator = "[";
		StringBuffer result = new StringBuffer();
		String temp = "";
		String tempClean = "";

		int index = exampleCode.indexOf(beginSeparator);
		temp = exampleCode.substring(0,index);
		tempClean = temp.trim(); // Otetaan whitespace pois

		if(!tempClean.equals("")){
		result.append("<textarea cols =\"" +this.COLS +"\""
									 +"rows =\"" +this.ROWS +"\" readonly>");
		result.append(tempClean);
		result.append("</textarea><br>");
		}

		return new String(result);
	}//getHTMLElementBeforeEmpty
	
	/**
	 * Luo sen osan mallikoodista, joka tulee aukkon jlkeen.
	 *
	 * @param exampleCode mallivastaus
	 * @return elementti html-muodossa
	 */  

	private String getHTMLElementAfterEmpty(String exampleCode){

		String endSeparator = "]";
		StringBuffer result = new StringBuffer();
		String temp = "";
		String tempClean = "";

		int index = exampleCode.indexOf(endSeparator);
		temp = exampleCode.substring(index+1);
		tempClean = temp.trim(); // Otetaan whitespace pois

		if (!tempClean.equals("")) { //  Testataan tuleeko aukon jlkeen koodia.
			result.append("<textarea cols =\"" +this.COLS +"\""
										 +"rows =\"" +this.ROWS +"\" readonly>");
			result.append(tempClean);
			result.append("</textarea><br>");
		}

				return new String(result);
				}//getHTMLElementAfterEmpty

	}//class
